--------------------------------------------------------------------------------
-- A standard circuit Hexadecimal to 7-segment decoder with active low outputs
-- PLAN B _ Behavioural approach. 
-- The truth table is written using the PROCESS statement
-- This is the translation of the schematic representing the truth table 
-- (Fig 5) https://digsys.upc.edu/csd/P02/HEX_7seg/Behav/img_planB.jpg in this
-- tutorial: https://digsys.upc.edu/csd/P02/HEX_7seg/P2_tut_BCD_7SEG_planB.html
--  
---------------------------------------------------
-- Project P2  - CSD :  standard logic circuits in VHDL
-- http://digsys.upc.edu
--------------------------------------------------------------------------------

LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;


ENTITY HEX_7SEG_DECODER IS

	PORT (
		X_IN   								: IN STD_LOGIC_VECTOR(3 DOWNTO 0); 
		BI_L   								: IN STD_LOGIC;      
		A_L, B_L, C_L, D_L, E_L, F_L, G_L  	: OUT STD_LOGIC	
		);
	  
END HEX_7SEG_DECODER;

ARCHITECTURE truth_table OF HEX_7SEG_DECODER IS

-- The schematic has an internal vector signal: 
SIGNAL  Y_OUT : STD_LOGIC_VECTOR (6 downto 0);

BEGIN
	PROCESS (BI_L, X_IN)  
		BEGIN
			IF BI_L = '1' THEN
				CASE X_IN IS    		
						-- 	Active-high outputs table 
						-- Xin(3,2,1,0)   abcdefg        
						-- ----        	  -------    
					WHEN  "0000" =>       		-- 0
								Y_OUT <= "1111110";
					WHEN  "0001" =>       		-- 1
								Y_OUT <= "0110000";
					WHEN  "0010" =>       		-- 2
								Y_OUT <= "1101101";
					WHEN  "0011" =>       		-- 3
								Y_OUT <= "1111001";
					WHEN  "0100" =>       		-- 4
								Y_OUT <= "0110011";
					WHEN  "0101" =>       		-- 5
								Y_OUT <= "1011011";
					WHEN  "0110" =>       		-- 6
								Y_OUT <= "1011111";
					WHEN  "0111" =>       		-- 7
								Y_OUT <= "1110000";
					WHEN  "1000" =>       		-- 8
								Y_OUT <= "1111111";-- All LED ON
					WHEN  "1001" =>       		-- 9
								Y_OUT <= "1110011";
					WHEN  "1010" =>       		-- A
								Y_OUT <= "1110111";
					WHEN  "1011" =>       		-- b
								Y_OUT <= "0011111";
					WHEN  "1100" =>       		-- C
								Y_OUT <= "1001110";
					WHEN  "1101" =>       		-- d
								Y_OUT <= "0111101";
					WHEN  "1110" =>       		-- E
								Y_OUT <= "1001111";
					WHEN Others =>       		-- F
								Y_OUT <= "1000111";
				END CASE;
			
			ELSE 
				Y_OUT <= "0000000"; -- Blank
			END IF;
	
	END PROCESS;
			
-- Extra logic (simple NOT buffers in this case to make 
-- active-low outputs )
-- 
A_L <= not(Y_OUT(6));
B_L <= not(Y_OUT(5));
C_L <= not(Y_OUT(4));
D_L <= not(Y_OUT(3));
E_L <= not(Y_OUT(2));
F_L <= not(Y_OUT(1));
G_L <= not(Y_OUT(0));

 	 		 		   	   
END truth_table;
